module Cache

where
import System.Directory
import Debug
import Monad
import System.Time
import GHC.IO
import GHC.Handle

cacheDir = "/ProjCache"

-- FIXME: expiration
isInCache :: String -> IO Bool
isInCache fn = do
    res <- doesFileExist fn
    debugMsg 1 (((if res then "exists!" else "not exists!")) ++ " " ++ fn)
    (if res then do
       clockTime <- getModificationTime fn
       curTime <- getClockTime
       debugMsg 1 $ "fn = " ++ fn ++ " diff = " ++ (show (diffClockTimes curTime clockTime))
-- expiration = 1 week
       return $ (diffClockTimes curTime clockTime) <= (TimeDiff 0 0  0 0 0 (1*(7*24*60*60)) 0)
     else
       return False)

 
getCachedData url = do
  nDebugMsg 1 $ "opening File: " ++ cacheFile url
  h <- openFile (cacheFile url) ReadMode
  c <- hGetContents h
  nDebugMsg 1 $ "about to close file"
  return (hClose h `seq` c)

addCachedData url res = do 
 let fn = cacheFile url
 debugMsg 1 "adding cached data" 
 recursivelyMakeDirectories "/" (drop 1 fn)
 writeFile fn res

cacheFile url = cacheDir ++ drop 6 url

recursivelyMakeDirectories context fn =
   let i = firstIndexOf '/' fn in do
     debugMsg 1 $ "fn = " ++ fn ++ " i = " ++ (show i)
     (if (i == -1) then
        return ()
      else 
        let parent = take i fn in do
           let todo = context ++ parent
           debugMsg 1 $ "creating " ++ todo
           ex <- doesDirectoryExist todo
           when (not ex) $ createDirectory todo
           recursivelyMakeDirectories (todo ++ "/") (drop ((length parent) + 1) fn))
 
firstIndexOf c xs = go 0 c xs
    where go n c [] = -1
          go n c (x:xs) | c == x = n
          go n c (x:xs) = go (n+1) c xs