module InterestsJunk where
import Control.Monad.State
import Util
import Text.XML.HaXml hiding (union)
import Text.XML.HaXml.Pretty
import Text.XML.HaXml.Xtract.Parse
import Data.Map hiding (union,map,filter)
import List
import Monad
import System (getArgs)
import GHC.IOBase
import System.Cmd
import System.Directory
import URLHandler
import Debug
import Char
--import MostTypicalFriends

printPairs typical = do
  mapM_ (\ (s,i) -> putStrLn $ s ++ " " ++ (show i) ++ "<br>") 
        typical

findMostTypical :: Map String (Int,[String]) -> IO [(String, Int)]
findMostTypical tab = do
  let typicalTable = foldr (\ person tab -> (addPerson tab person)) empty (concat (map (removeSingletons . snd) (elems tab)))
  return $ (sortBy (\ (_,x) (_,y) -> compare y x) (toList typicalTable))

removeSingletons l = (foldr (\ elt rest -> if (length elt) > 1 then (elt:rest) else rest) [] l)

addPerson :: Map String Int -> String -> Map String Int
addPerson tab person = 
  case (lookupFM tab person) of
    Just n -> addToFM tab person (n+1)
    Nothing -> addToFM tab person 0


getInterests :: String -> IO [(String,Int)]
getInterests user = do
  txt <- readTextFromURL (ljUrl ++ user)
  let intsToUse = (drop 2 (lines txt))
  debugMsg 3 $ "intsTouse = " ++ (show intsToUse)
  return $ zip (dropFirstTwoFields intsToUse) (map read (getField 1 intsToUse))
  
belowThreshold uthreshold lthreshold s = 
  let wds = words s in 
     (if ((length wds) < 3) then 
          False 
      else 
         let second = words s !! 1 in
           (if (isDigit (head second)) then 
              let numPeople = (read ((words s) !! 1)) in
                 numPeople < uthreshold && numPeople >= lthreshold
            else
                 False))

getFriendsFriends :: String -> FM [String]
getFriendsFriends user = do
  friends <- getFriends user 
  friendsfriends <- mapM getFriends friends
  return (user: (friends ++ (concat friendsfriends)))

getField :: Int -> [String] -> [String]
getField n lines =
       map (\ l -> (l !! n)) (getLinesWords lines)

dropFirstTwoFields :: [String] -> [String]
dropFirstTwoFields lines = map ( concatSpaces . (drop 2) ) (getLinesWords lines)

getLinesWords lines =   let linesWords :: [[String]] = map words lines in
                        filter ((> 2) . length) linesWords 
     
concatSpaces :: [String] -> String
concatSpaces xs = (concat (intersperse " " xs))

getSecondField :: [String] -> [String]
getSecondField lines = map (nth 1) (map words lines)

nth n l = (if (length l) <= n then "" else (l !! n))

ljUrl = "http://www.livejournal.com/misc/interestdata.bml?user="
fUrl = "http://www.livejournal.com/misc/fdata.bml?user="
outFileName = "/tmp/interests.txt"
mkFile user = "/tmp/interests_" ++ user ++ ".txt"

echo s = putStrLn s >> system s

beginsWith :: Char -> String -> Bool
beginsWith c (x:xs) | x == c = True
beginsWith _ _ = False
 
friendsFile user = "/tmp/friends_" ++ user ++ ".txt"

snd3 (_,a,_) = a

getFriends :: String -> FM [String]
getFriends commname  = do
  xmlData <- lift $ readTextFromURL (mkFoafURL commname)
  (if( "<?xml" `isPrefixOf` xmlData ) then do
        let xmlDoc = xmlParse "/dev/null" xmlData
        case xmlDoc of
           Document _ _ elt _ -> 
              let query = parseXtract "*//foaf:nick/-" in do
                 case elt of
                    Elem _ _ (_:(foafGroup:_)) -> do
                         let friends = sort $ (commname:(map show $ map content (query foafGroup foafGroup)))
                         setFriends commname friends
                         lift $ nDebugMsg 2 $ "FRENZ0RZ = " ++ (show friends)
                         return $ friends
                    Elem a b c -> do
                        lift $ putStrLn $ "name = " ++ a
                        --lift $ putStrLn $ "attr = " ++ (show b)
                        lift $ putStrLn $ "content = " ++ 
                              (concatMap (\x -> ("POOP: " ++ show x)) (map content c))
                        lift $ putStrLn $ "len = " ++ (show (length c))
--                        lift $ putStrLn $ "query = " ++ (show (map content (query c c)))
                        return []
   else  -- deleted journal
       return [])

mkFoafURL u = "http://www.livejournal.com/users/" ++ u ++ "/data/foaf" 

setFriends x y = do
  friendsMap <- getFriendsMap
  setFriendsMap (addToFM friendsMap x y)

getPersonFriends x = do
  friendsMap <- getFriendsMap 
  case (lookupFM friendsMap x) of
     Just y -> return y
     Nothing -> do
        setFriendsMap (addToFM friendsMap x [])
        return []

getFriendsMap :: FM (Map String [String])
getFriendsMap = get
setFriendsMap :: Map String [String] -> FM ()
setFriendsMap x = put x


type FM a = StateT (Map String [String]) IO a