import Util
import List
import Debug
import System.Environment
import Maybe
import System
import URLHandler
import InterestsJunk
import Data.Map hiding (map, filter)
import List
import Text.Regex
import Cache
import Monad
import Control.Monad.Trans
import Text.ParserCombinators.Parsec
import Text.ParserCombinators.Parsec.Error
import List
import Clustering
import Control.Monad.State
import XmlOutput
import Char
import Tags

-- Don't report interests with more than <threshold> users interested in them
defUpperThreshold = 100000000000000
-- Don't report interests with fewer than <threshold> users interested in them
defLowerThreshold = 1

defNumInts = 150


noMatch Nothing commname = True
noMatch (Just (_, s, t, _)) commname = s /= commname || t /= ""


main = do
  (arg:_) <- getArgs
-- getEnv( "QUERY_STRING" )
  let cachedDataUrl = mkOutputUrl arg
  cached <- isInCache cachedDataUrl
  (if cached then do
        res <- getCachedData $ cachedDataUrl
        putStrLn res
   else do
-- Only works for users now, not communities
-- TODO: want to use friendsfriends eventually
     (_,_) <- runStateT (doCrap arg) empty
     return ())

doCrap arg =     do
        friends <- getFriends arg
        lift $ nDebugMsg 1 $ "friends = " ++ (show friends)
        fff <- getFriendsFriends arg
        interests <- lift $ getInterests arg
        let friendsfriends = sort (nub friends)
        _ <- lift $ mapM getInterests friendsfriends
        let numFriends = length friendsfriends
        let ff = sort (nub fff) 
        lift $ nDebugMsg 1 $ "ff = " ++ (show ff)
        let ((userMap::UserMap),_) = mkUserMap ff
--        lift $ nDebugMsg 1 $ "userMap = " ++ (show (toList userMap))
        assertM $ (numElts userMap) == (length ff)
        (interestsMap,interestsTable) <- lift $ mkInterestsMap friendsfriends userMap
        lift $ putStrLn $ "User = " ++ arg ++ " friendsfriends = " ++ 
                   (show friendsfriends) ++ " interests = " ++ (show interests) ++ 
                   "\nNumber of friendsfriends: " ++ (show numFriends)
        lift $ nDebugMsg 1 $ "Outputting friends graph..."
        outputFriendsGraph arg userMap friendsfriends -- Should be friends not friendsfriends
        lift $ nDebugMsg 0 $ "Clustering by interests..."
        (hierRes,_) <- lift $ runStateT  (cluster friendsfriends) (interestsMap, userMap, interestsTable, empty, empty)
        lift $ putStrLn $ "Clustering results: " ++ (concatMap (\r -> (show r) ++ "\n") hierRes)
        -- cluster by friends
        friendsMap <- getFriendsMap
        encMap <- encodeFriendsMap userMap friendsMap
        lift $ nDebugMsg 0 $ "Clustering by friends..."
        (friendsClusteringRes,_) <- lift $ runStateT (cluster friendsfriends) (getIdsToFriendsMap userMap, userMap, encMap, empty, empty)
        lift $ putStrLn $ "Clustering results: " ++ (concatMap (\r -> (show r) ++ "\n") friendsClusteringRes)
   ---------------------------------------------------------
        lift $ nDebugMsg 0 $ "Doing k-means clustering..."
        (res,_) <- lift $ runStateT (kMeansCluster friendsfriends) (interestsMap, userMap, interestsTable, empty, empty)
        lift $ nDebugMsg 0 $ "K-means clustering results (interests): " ++ (show res)
        (fRes,_) <- lift $ runStateT (kMeansCluster friendsfriends) (getIdsToFriendsMap userMap, userMap, encMap, empty, empty)
        lift $ nDebugMsg 0 $ "K-means clustering results (friends): " ++ (show res)

        -- FIXME: commented out b/c it runs out of heap space
        --let friendsfriends = sort (nub fff)
       {- lift $ nDebugMsg 1 $ "fff = " ++ (show friendsfriends) -}

        ((tagRes,tagMap):(linkRes,linkMap):(ljUsersRes,ljUsersMap):_) <- lift $ clusterOnFeatures [("tags",getTags),("links",getLinks),("ljUsers",getLjUsers)] userMap friendsfriends 
        let decodedInterests =  decodeMap userMap interestsMap interestsTable
        let decodedFriends = decodeMap userMap (getIdsToFriendsMap userMap) encMap
        lift $ outputClusterFile arg numFriends 
                [("hierarchical", "interests", (ClusterGroup hierRes), decodedInterests),
                 ("hierarchical", "friends", (ClusterGroup friendsClusteringRes), decodedFriends),
                 ("kmeans", "interests", (Clusters res), decodedInterests),
                 ("kmeans", "friends", (Clusters fRes), decodedFriends),
                 ("kmeans", "tags", (Clusters tagRes), tagMap),
 		 ("kmeans", "links", (Clusters linkRes), linkMap),
                 ("kmeans", "lj users", (Clusters ljUsersRes), ljUsersMap)]
                [("interests", decodedInterests),
		 ("friends", decodedFriends),
		 ("tags", tagMap),
		 ("links", linkMap),
		 ("lj users", ljUsersMap)]
        return ()

clusterOnFeatures features userMap users = do
  (res,_) <- runStateT (mapM (\ (featName,featFun) -> 
                          clusterOnFeature featName userMap users featFun)
                        features) empty
  return res

clusterOnFeature :: String -> UserMap -> [String] -> ([String] -> XM (Map String [String])) -> XM ([[String]],Map String [String])
clusterOnFeature whatFeature userMap users extractFeatures = do
        features <- extractFeatures users
        lift $ nDebugMsg 1 $ "Features = " ++ (show features)
        let featureIdMap = mkFeatureMap features
        let encFeatureMap = encodeFeatureMap userMap featureIdMap features
        -- ignore users with no tags
        let usersToUse = deleteUsersWithNo users features
        lift $ nDebugMsg 0 $ "Users to use = " ++ (show usersToUse)
        (res,_) <- lift $ runStateT (kMeansCluster usersToUse) 
                      (invert featureIdMap, userMap, encFeatureMap, 
                         empty, empty)
        lift $ nDebugMsg 0 $ "K-means clustering results (" ++ whatFeature ++ "): " ++ (show res)
    	return (res,features)

mkFeatureMap :: Map String [String] -> Map String Int
mkFeatureMap m = 
   let allFeatures = nub (concat (elems m)) in
      Data.Map.fromList (zip allFeatures [1..])

encodeFeatureMap :: UserMap -> Map String Int -> Map String [String] -> Map Int [Int]
encodeFeatureMap userMap keys m = foldl (\ doneMap (user,features) -> addToFM doneMap (fromJust (personToId userMap user)) (map (\ feature -> fromJust (lookupFM keys feature)) features)) empty (Data.Map.toList m)

deleteUsersWithNo :: [String] -> Map String [String] -> [String]
deleteUsersWithNo users m = 
  filter (\ user -> 
    case (lookupFM m user) of
      Just (x:_) -> True
      _ -> False) users

encodeFriendsMap :: UserMap -> Map String [String] -> FM (Map Int [Int])
encodeFriendsMap userMap friendsMap = do
  let pairs::[(String,[String])] = toList friendsMap
  (res::[(Int,[Int])]) <- (foldM (dostuff userMap)  [] pairs)
  return $ fromList res

dostuff :: UserMap -> [(Int,[Int])] -> (String,[String]) -> FM [(Int,[Int])]
dostuff userMap sofar (fId, friendsIds) = do
    let (newFId::Int) = (myFromJust "encodeFriendsMap"
                           (personToId userMap fId))
    let (newIds::[Int]) = (map (\ x -> (myFromJust "encodeFriendsMap"
                             (personToId userMap x))) friendsIds)
    return ((newFId, newIds):sofar)

mkOutputUrl :: String -> String
mkOutputUrl user = cacheDir ++ "/" ++ user ++ "_friends"



mkInterestsMap :: [String] -> UserMap -> IO (Map Int String, Map Int [Int])
mkInterestsMap users userMap = 
   foldM (\ (interestToIdMap, userToInterestsMap) user -> do
               let interestsFile = ljUrl ++ user
               interests <- snarfInterests interestsFile
               let userId = (myFromJust "mkInterestsMap") $ (personToId userMap user)
               return(addToIdMap interestToIdMap interests, addToFM userToInterestsMap userId (map fst interests))) (empty,empty) users

addToIdMap :: Map Int String -> [(Int,String)] -> Map Int String
addToIdMap mapSoFar interests =
   foldl (\ intsMap (intid,interest) -> addToFM intsMap intid (dropFirst interest)) mapSoFar interests

snarfInterests :: String -> IO [(Int,String)]
snarfInterests interestsUrl = do
   txt <- readTextFromURL interestsUrl
   res <-   (mapM (\ (s::String) -> do
                       let  spaceInd = indexOf ' ' s
                       let idStr = (substring s 0 spaceInd)
                       nDebugMsg 5 $ "reading " ++ idStr
                       (if ((length idStr) > 0 ) && ( all isDigit (substring s 0 spaceInd) ) then do
                          let intId::Int = read $ substring s 0 spaceInd
                          nDebugMsg 5 $ "intId = " ++ (show intId)
                          let spaceInd2 = (indexOf ' ' (substring s spaceInd (length s))) 
                          return (intId, (drop spaceInd s)) 
                        else
                          return ((-1),""))) (lines txt))
   return $ (filter (\ (x,_) -> x > 0) res)
    
substring :: String -> Int -> Int -> String
substring s start end = (take end (drop start s))
indexOf :: Char -> String -> Int
indexOf c (c1:s) | c == c1 = 0
indexOf c [] = -1
indexOf c (s:ss) = 
   let i = indexOf c ss in
      (if i == (-1) then i else (i+1))

mkUserMap :: [String] -> (UserMap,Int)             
mkUserMap users = 
-- just make up user ids
   foldl (\ (userMap,n) user -> (addMapping userMap (n+1) user,n+1)) (emptyUserMap,0) users
