module Tags where
import Maybe (fromJust)
import Control.Monad.State
import Util
import Text.Regex.Posix
import Text.XML.HaXml hiding (union)
import Text.XML.HaXml.Pretty
import Text.XML.HaXml.Posn
import Text.XML.HaXml.Types
import Text.XML.HaXml.Xtract.Parse
import Data.Map hiding (union,map,filter)
import List
import Monad
import System (getArgs)
import GHC.IOBase
import System.Cmd
import System.Directory
import URLHandler
import Debug
import Char
import Data.Map hiding(map)

getTags :: [String] -> XM (Map String [String])
-- given a list of users, returns a table mapping users
-- to a list of tags they use
getTags users = getUserFeature getUserTags users

getUserFeature f users =
          foldM (\ mapSoFar user -> do
                           features <- f user
                           return $ addToFM mapSoFar user (nub features)) empty users

getLjUsers user = do
   users <- getUserFeature getLjUserLinks user
   lift $ nDebugMsg 1 $ "lj users mentioned by " ++ (show user) ++ " = " ++ (show users)
   return users

getLinks :: [String] -> XM (Map String [String])
getLinks = getUserFeature getAllUserLinks

getUserTags user = do
   l <- getXmlQuery user "category"
   return $ sort $ nub $ l

getXmlQuery :: String -> String -> XM [String]
getXmlQuery user tag = do
   dataCached <- isXmlCached user
   xmlDoc <- (if dataCached then
                 getXmlDoc user
              else do
                 xmlData <- lift $ readTextFromURL (rssUrl user)
                 lift $ nDebugMsg 1 $ "fetching data for " ++ (show user)
                 lift $ nDebugMsg 3 $ "data for user = " ++ (show user) ++ " = " ++ (show xmlData)
                 (if( "<?xml" `isPrefixOf` xmlData ) then do
                      lift $ nDebugMsg 1 $ "Parsing data for " ++ (show user)
                      return $ {-# SCC "xmlParse" #-} xmlParse "/dev/null" xmlData
                  else
                      return $ Document (Prolog Nothing [] Nothing []) [] (Elem "" [] []) []))
   (if dataCached then return () else setXmlDoc user xmlDoc)
   case xmlDoc of
         Document _ _ elt _ ->
            let query = {-# SCC "parseXtract" #-} parseXtract ("*//" ++ tag ++ "/-") in do
                case elt of
                   Elem _ _ stuff -> do
                     lift $ nDebugMsg 2 $ " stuff = " ++ (show (map content stuff))
                     let tags = (map show $ (map content (concatMap (\ s -> query s s) stuff)))
                     lift $ nDebugMsg 2 $ "Tags for user " ++ (show user) ++ " = " ++ (show tags)
                     return tags
 
getPrefixed prefix user = do
-- TODO: Would prefer to do something like getting rid of very common
-- links (userinfo.gif) and maybe just using sites rather than full URLs
   fullTextPosts <- getXmlQuery user "description"
   let urls = (List.filter (\x -> prefix `isPrefixOf` x) fullTextPosts)
   lift $ nDebugMsg 1 $ "urls = " ++ (show urls)
   return urls

getLjUserLinks user = do
 let prefix = "http://www.livejournal.com/userinfo.bml?user=" 
 links <- getPrefixed prefix user
 return (nub (map (drop (length prefix)) links))
 
getAllUserLinks user = getPrefixed "http://" user

rssUrl user = "http://www.livejournal.com/users/" ++ user ++ "/data/rss"

type XM a = StateT (Map String (Document Text.XML.HaXml.Posn.Posn)) IO a

isXmlCached user = do
  m <- getUserXmlMap
  return( user `member` m )

getXmlDoc :: String -> XM (Document Text.XML.HaXml.Posn.Posn)
getXmlDoc user = do
  m <- getUserXmlMap
  return $ fromJust (lookupFM m user)

setXmlDoc :: String -> Document Text.XML.HaXml.Posn.Posn -> XM ()
setXmlDoc user doc = do
  m <- getUserXmlMap
  setUserXmlMap (addToFM m user doc)

getUserXmlMap = get
setUserXmlMap = put
