{- read binary data:
hGetBuf
hSetBinaryMode
-}

module URLHandler where
import System.Posix.Unistd
import Monad
import Network
import Network.URI
import System
import System.Exit
import Util
import GHC.Handle
import System.IO 
import List ( find, isPrefixOf )
import Text.Regex
import Maybe ( isJust )
import Foreign.C.Types
import Control.Monad.Trans
import GHC.Ptr
import System.IO.Error
import Char
import Debug
import Cache

{-#include "star.h"-}
foreign import ccall "malloc" malloc :: CInt -> IO (Ptr CInt)

cSIZEOFCHAR = 8
ljMAXIMAGESIZE = 40000

readTextFromURL :: String -> IO String
readTextFromURL url = do
   cached <- isInCache (cacheFile url)
   nDebugMsg 1 $ "url " ++ url ++ " (file is " ++ (cacheFile url) ++ ") " ++ (if cached then "is" else "is not") ++ " cached"
   (if cached then
      (getCachedData url)
    else do
      h <- getSocket
      nDebugMsg 1 $ "sending to host: " ++ url
      hPutStrLn h ("GET " ++ url ++ " HTTP/1.1\r\nConnection: Keep-Alive\r\nUser-Agent: Information Visualization project; contact: catamorphism@livejournal.com\r\n\r\n")
      hFlush h
      res <- catch (hGetContentsHttp h) (\ e -> if isEOFError e then readTextFromURL url else ioError e) 
      debugMsg 5 $ "got from host: " ++ res
      addCachedData url res
      return res)

killHeaders h = do
   line <- hGetLine h
   debugMsg 3 $ "line = " ++ (show (map ord line))
   (if (line == [chr 13]) then
      return ()
    else
      killHeaders h)

mkTmpFile url = do
     case (parseURI url) of 
       Nothing ->  error $ "Bad URL " ++ url 
       Just uri -> do
          let fname = tmpDir ++ sep ++ (flatten (uriPath uri)) -- ++ "." ++ ext
          _ <- system( "cd " ++ tmpDir ++ "; wget " ++ url ++ "; mv " ++ (lastPart (uriPath uri)) ++ " " ++ fname)-- ++ " -O " ++ fname)
          return fname

lastPart :: String -> String
lastPart s = reverse (takeWhile ((/=) '/') (reverse s))

flatten :: String -> String
flatten path = map (\ c -> if c == '/' then '_' else c) path

dumpToFile :: String -> (Ptr CInt,Int) -> IO ()
dumpToFile fileName (buf,bufSize) = do
   h <- openFile fileName WriteMode
   hSetBinaryMode h True
   hPutBuf h buf bufSize
   hClose h

friendsListURL :: String -> String
friendsListURL user = "http://www.livejournal.com/misc/fdata.bml?user=" ++ user

getImageFileFor :: String -> Bool -> IO (Maybe String)
getImageFileFor friend saveImage = do
   let infoURL = "http://www.livejournal.com/allpics.bml?user=" ++ friend
   allpicsPage <- readTextFromURL infoURL
   nDebugMsg 5 $ "Friend = " ++ friend ++ " allpicsPage = " ++ allpicsPage
   matched <- (matchRegexMulti (mkRegex ("<img src='([^']+)' width='[0-9]+' height='[0-9]+' alt='[^']*' border='0'/>"))
                    allpicsPage)
   case matched of
      Just icons -> do 
         nDebugMsg 1 $ "Friend = " ++ friend ++ " userpics = " ++ (show icons)
	 [rand] <- randomElts 1 (\ _ -> return True) icons  
         saveURLToFile rand saveImage          
      Nothing -> (nDebugMsg 1 $ ("Friend = " ++ friend ++ " no userpics"))
                 >> return Nothing -- no userpics

saveURLToFile :: String -> Bool -> IO (Maybe String)
saveURLToFile url saveImage = do
   let uri = parseURI url
   case uri of
     Nothing -> return Nothing -- invalid URL
     Just uriDesc -> do
        fName <- (if saveImage then do
                      --res@(imageData,sz) <- readBinaryFromURL url
                      --nDebugMsg 1 $ "sz = " ++ (show sz)
                      fName <- mkTmpFile url -- imageData
                      --dumpToFile fName res
-- creating the file can fail -- should check for that
                      return fName
                    else
                      return "x")
        case fName of
           "" -> return Nothing
           _ -> return $ Just fName

tempDir = "/tmp/.icons"

fileSep = "/"
pathSep = '/'

hasIcons :: String -> IO Bool
hasIcons friend = do
  nDebugMsg 1 $ "hasIcons: checking " ++ friend
  res <- getImageFileFor friend False
  nDebugMsg 1 $ "hasIcons: getImageFileFor returned " ++ (show res)
  return (isJust res)

readBinaryFromURL url = do
   h <- getSocket
   hPutStrLn h ("GET " ++ url ++ " HTTP/1.1\r\n\r\n")
   hFlush h
   killHeaders h
   hSetBinaryMode h True
   buf <- malloc ( fromIntegral (cSIZEOFCHAR * ljMAXIMAGESIZE)::CInt )
   bytesRead <- hGetBuf h buf ljMAXIMAGESIZE 
   hSetBinaryMode h False
   return (buf,bytesRead)

