module Util where
import Maybe ( fromJust )
import Data.IntSet ( insert )
import Data.Map hiding (delete, map)
import Monad
import System.Random
import Prelude hiding (lookup)
import List ( find, isPrefixOf, delete ) 
import Text.Regex
import Network
import System.IO
import Debug
import Control.Exception

hostName = "www.livejournal.com"
portName = Service "http"

tmpDir = "/tmp/.icons"
sep = "/"

repeatNM :: Int -> IO a -> IO [a]
repeatNM 0 action = return []
repeatNM n action = do
  x <- action
  rest <- (repeatNM (n-1) action)
  return $ x:rest

scramble [] = return []
scramble xs = do
   [randomElt] <- randomElts 1 (\_ -> return True) xs
   rest <- scramble (delete randomElt xs)
   return $ randomElt:rest

group :: Int -> [a] -> [[a]]
group n l = go n l [[]]
   where  groupSize = (length l) `div` n
          go :: Int -> [a] -> [[a]] -> [[a]]
          go 0 l res = res
          go n [] res = res
          go n l@(x:xs) resultSoFar = 
           if( length (last resultSoFar) == groupSize ) then
              go (n - 1) xs (resultSoFar ++ [[x]])
           else
              let numSoFar = length resultSoFar in
                 let doneList = (take (numSoFar - 1) resultSoFar) in 
                    go n xs (doneList ++   
                               [((head (drop (numSoFar - 1) resultSoFar)) ++ [x])])
 
pickRandoms :: Int -> Int -> Int -> (a -> IO Bool) -> [a] -> IO [Int]
pickRandoms 0 _ _ _ _ = return []
pickRandoms numRandoms start end pred lst = do
   nDebugMsg 1 $ "pickRandoms: start = " ++ (show start) ++ " end = " ++ (show end)
   restOfRandoms <- pickRandoms (numRandoms - 1) start end pred lst
   newRandom <-  (getStdRandom (randomR (start,end)))
   ok <- pred (lst !! newRandom)
   (if( newRandom `elem` restOfRandoms 
    || (not ok)) then do
         newRandom <- pickRandoms 1 start end pred lst
         return (newRandom ++ restOfRandoms)
    else
         return (newRandom:restOfRandoms))

lastAfter :: Char -> String -> String
lastAfter c s = reverse (takeWhile ((/=) c) (reverse s))

data Flag = Width Int | Height Int
   deriving Show

useDefault :: (Flag -> Bool) -> a -> (Flag -> a) -> [Flag] -> a
useDefault pred defaultVal getter opts = 
    case (find pred opts) of
      Just opt -> getter opt
      Nothing -> defaultVal

isHeight (Height _) = True
isHeight _ = False

isWidth (Width _) = True
isWidth _ = False

getHeight ~(Height x) = x

getWidth ~(Width x) = x

matchRegexMulti regex str = do
  case (matchRegexAll regex str) of
    Just (_,_,_,[]) -> do
      debugMsg 5 $ "str = " ++ str ++ " no subexp matches" 
      (return $ Just [])
    Just (beforeMatch, matched, afterMatch, subexpMatches) -> do
      debugMsg 5 $ "str = " ++ str ++ " matched = " ++ matched ++ " afterMatch == " ++ afterMatch
      rest <- (matchRegexMulti regex afterMatch)
      return $ Just $ subexpMatches ++ (case rest  of
                                Nothing -> []
                                Just foo -> foo)
    Nothing -> return Nothing

randomElts :: Int -> (a -> IO Bool) -> [a] -> IO [a]
randomElts num pred lst = do
   nDebugMsg 1 $ "randomElts: num = " ++ (show num)
   inds <- pickRandoms num 0 ((length lst) - 1) pred lst
   nDebugMsg 2 $ "randomElts: inds = " ++ (show inds)
   return $ map (\ i -> lst !! i) inds

getSocket :: IO Handle
getSocket = do {-
  (maybeHandle,s,st) <- getSt
  case maybeHandle of
     Just h -> do
       assertIsOpen h
     {-  isEof <- hIsEOF h
       (if isEof then do
          putSt (Nothing,s,st)
          getSocket
        else -}
       return h --)
     Nothing -> do -}
        h <-  connectTo hostName portName
        hSetBuffering h LineBuffering
      --  putSt ((Just h),s,st)
        return h

hGetContentsHttp :: Handle -> IO String
hGetContentsHttp h = do
          assertIsOpen h
          line <- (hGetLine h) 
          debugMsg 2 $ line
          (if ("Content-length" `isPrefixOf` line) then do
             let len::Int = read ((words line) !! 1)
             debugMsg 2 $ "len = " ++ (show len)
             readTillNewLine h
             res <- hReadBytes h len  
             debugMsg 2 $ res 
             return res
           else
             hGetContentsHttp h)

hReadBytes :: Handle -> Int -> IO String
hReadBytes _ 0 = return ""
hReadBytes h n = do
   c <- hGetChar h
   rest <- hReadBytes h (n - 1)
   debugMsg 5 $ "n = " ++ (show n)
   return $ c:rest

fact 0 = 1
fact n = n*(fact (n - 1))


assertIsOpen h = do
   isOpen <- hIsOpen h
--   isEof <- hIsEOF h
   (if isOpen then
       debugMsg 1 $ "handle is open"
    else 
       debugMsg 1 $ "handle is not open")
  {- (if isEof then
       debugMsg 1 $ "isEof = true"
    else
       debugMsg 1 $ "isEof = false")
-}
readTillNewLine h = do
   l <- (liftM length) $ hGetLine h
   debugMsg 1 $ "length: " ++ (show l)
   (if l > 2 then
       readTillNewLine h 
    else
       return ())


myFromJust msg (Just x) = x
myFromJust msg Nothing = error msg

assertM b | b = return ()
assertM b = error $ "Assertion failed"


addToFM m k v = Data.Map.insert k v m
lookupFM k m = lookup m k

type UserMap = (Map Int String,Map String Int)

numElts :: UserMap -> Int
numElts (m,_) = length (keys m)

personToId :: UserMap -> String -> Maybe Int
personToId (_,m) x = lookupFM m x

idToPerson :: UserMap -> Int -> Maybe String
idToPerson (m,_) x = lookupFM m x

getIdsToFriendsMap :: UserMap -> Map Int String
getIdsToFriendsMap = fst

addMapping :: UserMap -> Int -> String -> UserMap
addMapping (intToPerson,personToInt) i s = (addToFM intToPerson i s,addToFM personToInt s i)

emptyUserMap :: UserMap
emptyUserMap = (empty,empty)

iterateM :: Monad m => (a -> m a) -> a -> Int -> m [a]
iterateM f x 0 = return []
iterateM f x n = do
   res <- f x
   rest <- iterateM f res (n-1)
   return $ res:rest


insertInto clusterIndex user clustersSoFar = 
  let i = clusterIndex 
      before = take (i-1) clustersSoFar
      rightOne = clustersSoFar !! (i-1)
      after = drop i clustersSoFar in
    before ++ [(Data.IntSet.insert user rightOne)] ++ after


invert :: Ord b => Map a b -> Map b a
-- assumes this is a bijection
invert m = Data.Map.fromList (zip (elems m) (keys m))

decodeMap :: UserMap -> Map Int String -> Map Int [Int] -> Map String [String]
decodeMap userMap featureDecoderMap featureMap =
   foldl doStuff empty 
       (Data.Map.toList featureMap)
    where doStuff :: Map String [String] -> (Int,[Int]) -> Map String [String]
          doStuff mapSoFar (user,features) =
              addToFM mapSoFar (fromJust (idToPerson userMap user))
                        (map (fromJust . (lookupFM featureDecoderMap)) features)


dropFirst feature = foldl (\ rest l -> rest ++ (if( rest /= "") then " " else "") ++ l) 
                      ""
                       (drop 1 (words feature))
