module XmlOutput

where
import URLHandler
import Text.ParserCombinators.Parsec
import Char
import List
import Control.Monad.State
import Debug
import Util
import Monad
import InterestsJunk
import Data.Map hiding (map)
import Maybe
import Cache

outputFriendsGraph :: String -> UserMap -> [String] -> FM ()
outputFriendsGraph mainUser userMap people = do
   (nodes,_,edges) <- foldM (doStuff userMap)       ([], [], []) people
   lift $ nDebugMsg 3 $ "Writing file: " ++ (mkOutputFileName mainUser)
   lift $ nDebugMsg 2 $ "nodes = " ++ (show nodes)
   lift $ nDebugMsg 2 $ "edges = " ++ (show edges)
   lift $ writeFile (mkOutputFileName mainUser) ( "<graph directed=\"1\">\n" ++ (unlines nodes) ++ (unlines edges) ++ "</graph>")
   lift $ nDebugMsg 3 $ "Wrote file!"

doStuff :: UserMap -> ([String],[String],[String]) -> String -> FM ([String],[String],[String])
doStuff userMap (nodes,existingNodes,edges) person = do
-- TODO: doesn't add nodes for friends
              lift $ nDebugMsg 3 $ "doStuff: person = " ++ person
              (friendnames::[String]) <- getPersonFriends person
              lift $ nDebugMsg 3 $ "doStuff: friends = " ++ (show friendnames)
              let friends = encode userMap friendnames
              lift $ nDebugMsg 3 $ "doStuff: friends = " ++ (show friends)
              let makeNewNode = not (person `elem` existingNodes)
              lift $ nDebugMsg 3 $ "makeNewNode = " ++ (show makeNewNode)
              let newExistingNodes = (if makeNewNode then person:existingNodes else existingNodes)
              lift $ nDebugMsg 3 $ "newExistingNodes = " ++ (show newExistingNodes)
              let uid = (myFromJust ("doStuff: " ++ (show uid)) (personToId userMap person))
              lift $ nDebugMsg 3 $ "uid = " ++ (show uid)
              userPicUrl <- lift $ getUserPicUrl person 
              let newNodes = (if makeNewNode then ("<node id=\"" ++ (show uid) ++ "\" label=\"" ++ person ++ "\" userpic=\"" ++ userPicUrl ++ "\"></node>"):nodes else nodes)
              lift $ nDebugMsg 3 $ "newNodes = " ++ (show newNodes)
              let newEdges = (map (\ friend -> "<edge label=\"\" source=\"" ++ (show uid) ++ "\" target=\"" ++ show friend ++ "\"></edge>") friends) ++ edges
              lift $ nDebugMsg 3 $ "newEdges = " ++ (show newEdges)
              lift $ nDebugMsg 3 $ "doneStuff: person = " ++ person
              return (newNodes, newExistingNodes, newEdges)
   
mkOutputFileName mainUser = cacheDir ++ "/prefuseGraphs/" ++ mainUser

encode :: UserMap -> [String] -> [Int]
encode userMap friendNames = map (\ x -> ((myFromJust ("encode: " ++ x)) (personToId userMap x))) friendNames

outputClusterFile userName numUsers stuff tables = do
  clusterings <- mapM (outputClusterFileOne userName numUsers) stuff
  let result = "<clusterings>\n" ++ (concat clusterings) ++ "\n" ++
                 (showTables tables) ++ "\n</clusterings>"
  let outFile = cacheDir ++ "/prefuseClusterFiles/" ++ userName
  writeFile outFile result

outputClusterFileOne :: String -> Int -> (String, String, ClusteringResult, Map String [String]) -> IO String
outputClusterFileOne userName numUsers (algorithm, feature, res, featureMap) = do
  -- find the list that has the number of clusters we want:
  -- about n/5
  clusterListToUse <-
    case res of
      ClusterGroup clusterGroups -> do 
          let numGroups = length clusterGroups
          let numClustersToUse = numUsers`div`5
          let lengths = map length clusterGroups
          nDebugMsg 1 $ "numClustersToUse = " ++ (show numClustersToUse) ++ " lengths = " ++ (show lengths)
          let clusterListToUse = clusterGroups !! ((numGroups - numClustersToUse) - 1)
          return clusterListToUse
      Clusters clusters -> return clusters
  nDebugMsg 0 $ "clusterListToUse = " ++ (show clusterListToUse)
  let result = "<clusters algorithm=\'" ++ algorithm ++ "\' feature=\'" ++ feature ++ "\'>\n" ++ (concatMap (showCluster featureMap) clusterListToUse) ++ "\n</clusters>"
  return result
  
showCluster featureMap oneCluster = "<cluster>\n" ++ (concatMap (\ u -> "<user name=\"" ++ u ++ "\"></user>\n") oneCluster) ++ (showFeatures featureMap oneCluster) ++ "</cluster>\n"

showFeatures featureMap cluster = 
  let featureToUsers = 
       flipTable (Just cluster) featureMap in
    concatMap (\ (feature,users) -> 
                 if( (length users) > 1
-- totally punt on anything that's an HTML entity
                    && (not (badXml feature))) then 
                      "<featureList name=\'" ++ (mangle feature) ++ "\'>\n" ++ (showUsers users)
                         ++ "</featureList>\n"
                 else
                      "" ) (Data.Map.toList featureToUsers)
      where showUsers users = concatMap (\ u -> "<user name=\'" ++ u ++ "\'></user>\n") users

showTables :: [(String,Map String [String])] -> String
showTables stringMapPairs = concatMap showTable stringMapPairs
   where showTable (label, table) = 
            "<featureType name=\'" ++ label ++ "\'>\n"
          ++ showUsers table
          ++ "\n</featureType>\n"
         showUsers table = 
           concatMap (\ (featureName,users) ->
		       (if (not (badXml featureName)) then
                        (
                         "<feature name=\'" ++ (mangle featureName) ++ "\'>\n"
                        
                ++ concatMap (\ u -> "<user name=\'" ++ u ++ "\'></user>\n") users
                ++ "</feature>\n")
			else "")) (Data.Map.toList (flipTable Nothing table))

getUserPicUrl person = do
  imgFile <- getImageFileFor person True
  case imgFile of
     Just x -> return x
     Nothing -> return ""


mangle feature = concatMap (\ c -> (if (c == '\'') then "&apos;" 
                                    else if( c == '&' ) then "&amp;"
		                    else if( c == '\"' ) then "&quot;"
                                    else [c])) feature

flipTable maybeCluster t = 
 foldl (\ mapSoFar (user,features) ->
              if( maybeCluster == Nothing || user `elem` (fromJust maybeCluster)) then
                     foldl (\ mapSoFar feature -> 
                           case (lookupFM mapSoFar feature) of
                              Nothing -> addToFM mapSoFar feature [user]
                              Just l -> addToFM mapSoFar feature (user:l)) mapSoFar features
	      else
                  mapSoFar
                  ) empty (Data.Map.toList t)

data ClusteringResult = ClusterGroup [[[String]]] | Clusters [[String]]


badXml feature = ((length feature) >= 2
             && ("&" `isPrefixOf` feature) && (last feature == ';'))
          || (not (all ((<= 128) . ord) feature))

userpicUrl person = "http://www.livejournal.com/allpics.bml?user=" ++ person