package prefuse.KirstenProject;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.TextField;
import java.awt.event.KeyListener;
import java.awt.event.KeyEvent;
import java.awt.Font;
import java.awt.Insets;
import java.awt.Paint;
import java.awt.PopupMenu;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.io.FileInputStream;
import java.awt.Button;

import java.util.*;

import javax.swing.JFrame;
import javax.swing.JRadioButton;
import javax.swing.ButtonGroup;
import javax.swing.JPanel;

import edu.berkeley.guir.prefuse.Display;
import edu.berkeley.guir.prefuse.EdgeItem;
import edu.berkeley.guir.prefuse.graph.DefaultEdge;
import edu.berkeley.guir.prefuse.graph.Edge;
import edu.berkeley.guir.prefuse.graph.Entity;
import edu.berkeley.guir.prefuse.graph.Node;
import edu.berkeley.guir.prefuse.ItemRegistry;
import edu.berkeley.guir.prefuse.NodeItem;
import edu.berkeley.guir.prefuse.graph.Edge;
import edu.berkeley.guir.prefuse.VisualItem;
import edu.berkeley.guir.prefuse.action.RepaintAction;
import edu.berkeley.guir.prefuse.action.assignment.ColorFunction;
import edu.berkeley.guir.prefuse.action.filter.GraphFilter;
import edu.berkeley.guir.prefuse.activity.ActionList;
import edu.berkeley.guir.prefuse.activity.Activity;
import edu.berkeley.guir.prefuse.event.ControlListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseWheelEvent;
import edu.berkeley.guir.prefuse.graph.Graph;
import edu.berkeley.guir.prefuse.graph.io.XMLGraphReader;
import edu.berkeley.guir.prefuse.render.DefaultEdgeRenderer;
import edu.berkeley.guir.prefuse.render.DefaultNodeRenderer;
import edu.berkeley.guir.prefuse.render.DefaultRendererFactory;
import edu.berkeley.guir.prefuse.render.TextItemRenderer;
import edu.berkeley.guir.prefusex.controls.DragControl;
import edu.berkeley.guir.prefusex.controls.FocusControl;
import edu.berkeley.guir.prefusex.controls.NeighborHighlightControl;
import edu.berkeley.guir.prefusex.controls.PanControl;
import edu.berkeley.guir.prefusex.controls.ZoomControl;
import edu.berkeley.guir.prefusex.force.DragForce;
import edu.berkeley.guir.prefusex.force.ForcePanel;
import edu.berkeley.guir.prefusex.force.ForceSimulator;
import edu.berkeley.guir.prefusex.force.NBodyForce;
import edu.berkeley.guir.prefusex.force.SpringForce;
import edu.berkeley.guir.prefusex.layout.ForceDirectedLayout;
/**
 * Application demo of a graph visualization using an interactive
 * force-based layout.
 *
 * @version 1.0
 * @author <a href="http://jheer.org">Jeffrey Heer</a> prefuse(AT)jheer.org
 */
public class KirstenProject extends Display implements ActionListener, KeyListener, ControlListener {

    private JFrame     frame;
    private InterestsPanel ipanel;
    public DetailsPanel dpanel;
    private TextField interestsField;
    private List highlightedUserList;
    public Pair currentFeaturePair;
    private PopupMenu itemMenu;
    private Button unhideAllButton;
    private final String menuStr = "Hide This User";
    private final String highlightStr = "Highlight This User";
    private final String clearStr = "Clear Highlighted Items";
    private VisualItem currentItem;
    private HashSet hiddenItems = new HashSet();
    public HashMap userpicMap;
    
    private Graph gr;
    
    private ForceSimulator m_fsim;
    private String         m_textField;
    public ItemRegistry   m_registry;
    private Activity       m_actionList;
    
    private Font frameCountFont = new Font("SansSerif", Font.PLAIN, 14);
    
    private HashMap featurePairToFakeEdges;
    
    private ForceDirectedLayout fdLayout;
    public HashMap  userToCluster;
    private HashMap clusterIdToUsers;
    public HashMap featureMap;
    public HashMap allFeaturesMap;
    private HashMap labelsToNodes;
    private ButtonGroup interestsFriendsButton;
    public HashSet highlightedItems;
    private int maxValue = -1;
    private String CLUSTER_TYPE = "cluster_type";
    
    public KirstenProject(Graph g, ForceSimulator fsim, HashMap utc, HashMap ctu, HashMap fm, HashMap afm,
    		HashMap upMap, HashMap lnMap) {
    	
    	this(g, fsim, utc, ctu, fm, afm, upMap, lnMap, "label");
    } //
    
    public KirstenProject(Graph g, ForceSimulator fsim, HashMap utc, HashMap ctu, HashMap fm, HashMap afm, 
    		HashMap upMap, HashMap lnMap, String textField) {
    	userToCluster = utc;
    	clusterIdToUsers = ctu;
    	featureMap = fm;
    	userpicMap = upMap;
    	gr = g;
    	allFeaturesMap = afm;
    	labelsToNodes = lnMap;
    	currentFeaturePair = new Pair( "kmeans", "interests");
        this.addControlListener( this );
    	// set up component first
        m_fsim = fsim;
        m_textField = textField;
        m_registry = new ItemRegistry(g);
        this.setItemRegistry(m_registry);
             
        fdLayout = new ForceDirectedLayout(m_fsim, false, false);
        addClusterEdges( g );
        updateEdges( currentFeaturePair.toString() );
        
        initRenderers();
        m_actionList = initActionList();
        setSize(500, 500);
        pan(350,350);
        this.addControlListener(new NeighborHighlightControl());
        this.addControlListener(new DragControl(false, true));
        this.addControlListener(new FocusControl(0));
        this.addControlListener(new PanControl(false));
        this.addControlListener(new ZoomControl(false));
    } //
    
    private void addClusterEdges( Graph g ) {
    	featurePairToFakeEdges = new HashMap();
    	
    	// only do this for currentFeaturePair for now 
    	Iterator allFeatureKeys = clusterIdToUsers.keySet().iterator();
    	while( allFeatureKeys.hasNext() ) {
    		String fPair = (String) allFeatureKeys.next();
    	HashSet newM = new HashSet();
    	featurePairToFakeEdges.put( fPair, newM );
    	HashMap m = (HashMap) clusterIdToUsers.get( fPair );
    	Iterator allClusterIds = m.keySet().iterator();
    	while( allClusterIds.hasNext() ) {
    		Integer clusterId = (Integer) allClusterIds.next();
    		List usersThisCluster = (List) m.get( clusterId );
    		Iterator iter1 = usersThisCluster.iterator();
    		// for each pair of users in cluster,
    		// add an edge between those users
    		while( iter1.hasNext() ) {
    			Iterator iter2 = usersThisCluster.iterator();
    			String user1 = (String) iter1.next();
    			Node node1 = null; // = (Node) labelsToNodes.get( user1 );
    			Iterator nodesIter1 = g.getNodes();
    			while( nodesIter1.hasNext() ) {
    				Node nextNode = (Node) nodesIter1.next();
    				if( nextNode.getAttribute( "label").equals( user1 ) ) {
    					node1 = nextNode;
    					break;
    				}
    			}
    			NodeItem nodeItem1 = (NodeItem) m_registry.getItem(
    					ItemRegistry.DEFAULT_NODE_CLASS, node1, false, false );
    			while( iter2.hasNext() ) {
    				String user2 = (String) iter2.next();
    				Node node2 = null; 
    				Iterator nodesIter2 = g.getNodes();
        			while( nodesIter2.hasNext() ) {
        				Node nextNode = (Node) nodesIter2.next();
        				if( nextNode.getAttribute( "label").equals( user2 ) ) {
        					node2 = nextNode;
        					break;
        				}
        			}
    				//= (Node) labelsToNodes.get( user2 );
    				//System.out.println( "user1 = " + user1 );
    				//System.out.println( "user2 = " + user2 );
    				//System.out.println( "node1 = " + node1 );
    				//System.out.println( "node2 = " + node2 );
    				NodeItem nodeItem2 = (NodeItem) m_registry.getItem(
        					ItemRegistry.DEFAULT_NODE_CLASS, node2, true, false );
    				
    				if( !(user1.equals( user2 ))
    				&& !(g.isNeighbor( user1, user2 ))) {
    				//	System.out.println( "Adding edge between " + user1 + " and " + user2 );
    					Edge e = new DefaultEdge( node1, node2, 
    							//nodeItem1, nodeItem2,
    							true );
    					/*System.out.println( "nodeItem1 = "
    							+ nodeItem1 + " nodeItem2 = " 
    							+ nodeItem2 );
    							*/
    				//	System.out.println( "e = " + e );
    					EdgeItem ei = (EdgeItem) m_registry.getItem( 
    							ItemRegistry.DEFAULT_EDGE_CLASS, e, true, false );
    					//g.addEdge( e );
    					newM.add( e );
    						System.out.println( "ei = " + ei );
    					fdLayout.setSpringCoefficient( ei, 
    							new Float( 4E-6f ) );
    					fdLayout.setSpringLength( ei, 
    							new Float( 100f ) );
    					ei.setAttribute( CLUSTER_TYPE, 
    							currentFeaturePair.toString() );
    					
    				}
    			}
    		}
    		
       	}
    	}
    }
    public void runDemo() {
        // now set up application window
       
        frame = new JFrame("Blog Clustering Demo");
        Container c = frame.getContentPane();
        c.setLayout(new BorderLayout());
        interestsField = new TextField();
        highlightedItems = new HashSet();
        highlightedUserList = new ArrayList();
        System.out.println( "Setting ifield size");
        interestsField.setSize( 20, 50 );
        System.out.println( "ifield size = " + interestsField.getSize());
        c.add( interestsField, BorderLayout.NORTH );
        System.out.println( "ifield size = " + interestsField.getSize());
        
        unhideAllButton = new Button( "Unhide All");
        unhideAllButton.addActionListener( this );
        c.add( unhideAllButton, BorderLayout.WEST );
        
        interestsField.addKeyListener( this );
        c.add(this, BorderLayout.CENTER);
        Container c1 = new Container();
        c1.setLayout( new BorderLayout() );
        c1.add( ipanel, BorderLayout.NORTH);
        c1.add( dpanel, BorderLayout.SOUTH );
        c.add(c1, BorderLayout.EAST);
        interestsFriendsButton = new ButtonGroup();
        JRadioButton b1 = new JRadioButton("Interests");
        JRadioButton b2 = new JRadioButton("Friends");
        JRadioButton b3 = new JRadioButton( "Tags" );
        JRadioButton b4 = new JRadioButton( "Links" );
        JRadioButton b5 = new JRadioButton( "LJ Users");
        b1.addActionListener( this );
        b2.addActionListener( this );
        b3.addActionListener( this );
        b4.addActionListener( this );
        b5.addActionListener( this );
        interestsFriendsButton.add( b1 );
        interestsFriendsButton.add( b2 );
        interestsFriendsButton.add( b3 );
        interestsFriendsButton.add( b4 );
        interestsFriendsButton.add( b5 );
        JPanel bPanel = new JPanel();
        bPanel.add( b1 );
        bPanel.add( b2 );
        bPanel.add( b3 );
        bPanel.add( b4 );
        bPanel.add( b5 );
        b1.setSelected( true );
        c.add( bPanel, BorderLayout.SOUTH );
        
        itemMenu = new PopupMenu();
        itemMenu.add( menuStr );
        itemMenu.add( highlightStr );
        itemMenu.add( clearStr );
        frame.add( itemMenu );
        itemMenu.addActionListener( this );
        
        frame.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                System.exit(0);
            }
        });
        frame.addComponentListener(new ComponentAdapter() {
            public void componentResized(ComponentEvent e) {
                Dimension d = frame.getSize();
                Dimension p = ipanel.getSize();
                Insets in = frame.getInsets();
                KirstenProject.this.setSize(d.width-in.left-in.right-p.width,
                        d.height-in.top-in.bottom);
            } //
            
        });
        frame.pack();
        frame.setVisible(true);
        
        // start force simulation
        m_actionList.runNow();
    } //
    
    public void updateEdges(String oldFeaturePair) {
    	HashSet oldEdgeSet = (HashSet) 
    		featurePairToFakeEdges.get( oldFeaturePair );
    	Iterator iter = oldEdgeSet.iterator();
    	while( iter.hasNext() ) {
    		gr.removeEdge( (Edge) iter.next() );
    	}
    	HashSet newEdgeSet = (HashSet)
    		featurePairToFakeEdges.get( currentFeaturePair.toString() );
    	iter = newEdgeSet.iterator();
    	while( iter.hasNext() ) {
    		gr.addEdge( (Edge) iter.next() );
    	}
    }
    public void actionPerformed(ActionEvent e) {
    	if( e.getSource() instanceof JRadioButton ) {
    	String feature = e.getActionCommand().toLowerCase();
    	String oldFeaturePair = currentFeaturePair.toString();
    	currentFeaturePair = new Pair( "kmeans", feature );
    	updateEdges( oldFeaturePair );
    	maxValue = -1;
    	}
    	else if( e.getSource() == itemMenu 
    			&& e.getActionCommand().equals( menuStr ) ) {
    		System.out.println( "MENU EVENT LOLOLOL");
    		System.out.println( e.getActionCommand() );
    		System.out.println( "currentItem = " + currentItem );
    		hiddenItems.add( currentItem );
    	}
    	else if( e.getSource() == itemMenu
    			&& e.getActionCommand().equals( highlightStr ) ) {
    		highlightedItems.add( currentItem );
    	}
    	else if( e.getSource() == itemMenu
    		&& e.getActionCommand().equals( clearStr ) ) {
    			highlightedItems.clear();
    		}
    	else if( e.getSource() == unhideAllButton ) {
    		hiddenItems.clear();
    	}
    }
    public void itemClicked( VisualItem item, MouseEvent e ) {
    	if ( item instanceof NodeItem 
    			&& e.getButton() == MouseEvent.BUTTON3 ) {
    		currentItem = item;
        	Entity e1 = m_registry.getEntity( item );
        	String label = ((Node) e1).getAttribute("label");
        	System.out.println( "Item right-clicked! " + label );
        	itemMenu.show( this, e.getX(), e.getY() );
    	}
    }
    public void itemDragged( VisualItem item, MouseEvent e ) {}
    public void itemEntered( VisualItem item, MouseEvent e ) {}
    public void itemExited( VisualItem item, MouseEvent e ) {}
    public void itemKeyPressed( VisualItem item, KeyEvent e ) {}
    public void itemKeyReleased( VisualItem item, KeyEvent e ) {}
    public void itemKeyTyped( VisualItem item, KeyEvent e ) {}
    public void itemMoved( VisualItem item, MouseEvent e ) {}
    public void itemPressed( VisualItem item, MouseEvent e ) {}
    public void itemReleased( VisualItem item, MouseEvent e ) {}
    public void itemWheelMoved( VisualItem item, MouseWheelEvent e ) {}
    public void mouseClicked( MouseEvent e ) {}
    public void mouseDragged(  MouseEvent e ) {}
    public void mouseEntered(  MouseEvent e ) {}
    public void mouseExited(  MouseEvent e ) {}
    public void mouseMoved(  MouseEvent e ) {}
    public void mousePressed(  MouseEvent e ) {}
    public void mouseReleased(  MouseEvent e ) {}
    public void mouseWheelMoved(  MouseWheelEvent e ) {}
    
    public void keyTyped( KeyEvent e) {
    	String s = interestsField.getText();
    	highlightedUserList.clear();
    	if( s != null && s.length() > 0 ) {
    		HashMap m = (HashMap) allFeaturesMap.get( currentFeaturePair.getSecond());
    	
    			String sToLookup = s; //+ "," + ((String)currentFeaturePair.getSecond()).toUpperCase();
        	//	System.out.println( "looking up " + s + " in " + m);
    			if( m.containsKey( sToLookup ) ) {
    				highlightedUserList.addAll( (List) m.get( sToLookup ) );
    			}
    	}
    }
    public void keyPressed( KeyEvent e) {
    	
    }
    public void keyReleased( KeyEvent e ) {
    	
    }
    private void initRenderers() {
        TextItemRenderer    nodeRenderer = new TextItemRenderer();
        nodeRenderer.setRenderType(TextItemRenderer.RENDER_TYPE_FILL);
        nodeRenderer.setRoundedCorner(8,8);
        nodeRenderer.setTextAttributeName(m_textField);
        DefaultNodeRenderer nRenderer = new DefaultNodeRenderer();
        DefaultEdgeRenderer edgeRenderer = new DefaultEdgeRenderer();    
        m_registry.setRendererFactory(new DefaultRendererFactory(
                nodeRenderer, edgeRenderer, null));
    } //
    
    private ActionList initActionList() {
        ActionList actionList = new ActionList(m_registry,-1,20);
        actionList.add(new GraphFilter());
        actionList.add( fdLayout );
        actionList.add(new DemoColorFunction());
        actionList.add(new RepaintAction());
        ipanel = new InterestsPanel( userToCluster, clusterIdToUsers, this );
        dpanel = new DetailsPanel( this );
        MyFocusListener fl = new MyFocusListener();
        fl.setIpanel( ipanel );
        actionList.add( fl );
        return actionList;
    } //
    
    public static void main(String argv[]) { 
    	String user = "jholomorphic";
        String file = (argv.length==0 ? "etc/" + user +".xml" : argv[0]);
        //String file = "../prefuse/etc/terror.xml";
        String clusterFile = "etc/" + user + "_clusters.xml";
        Graph g;
        XMLGraphReader r = new XMLGraphReader();
        try {
            g = r.loadGraph(file);
        } catch ( Exception e ) { e.printStackTrace(); return; }
        
        HashMap upMap = r.getUserpicMap();
        HashMap lnMap = r.getLabelNodeMap();
        System.out.println( "upMap = " + upMap );
        
        Pair p;
        XMLClusterReader r1 = new XMLClusterReader();
        try {
        	p = r1.loadClusters( new FileInputStream(clusterFile)); 	
        } catch ( Exception e ) { e.printStackTrace(); return; }
        HashMap userToCluster = (HashMap) p.getFirst();
        HashMap clusterIdToUsers = (HashMap) p.getSecond();
        
        System.out.println( "userToCluster = " + userToCluster.toString() );
        System.out.println( "clusterIdToUsers " + clusterIdToUsers.toString() );
  
        HashMap featureMap = r1.getClusterIdToFeaturesMap();
        System.out.println( "featureMap = " + featureMap );
        HashMap allFeaturesMap = r1.getFeatureMap();
        
        System.out.println("Visualizing Graph: "
            +g.getNodeCount()+" nodes, "+g.getEdgeCount()+" edges");
        
        ForceSimulator fsim = new ForceSimulator();
        fsim.addForce(new NBodyForce(-0.4f, -1f, 0.9f));
        fsim.addForce(new SpringForce(4E-7f, 100f));
        fsim.addForce(new DragForce(-0.015f));
        
        KirstenProject fdemo = new KirstenProject(g, fsim, userToCluster, clusterIdToUsers, featureMap, allFeaturesMap, upMap, lnMap );
        fdemo.runDemo(); 
    } //

    public class DemoColorFunction extends ColorFunction {
        private Color pastelRed = new Color(255,125,125);
        private Color pastelOrange = new Color(255,200,125);
        private Color lightGray = new Color(220,220,255);
        private Color lightGreen = new Color(144, 238, 144);
        private Color lightBlue = new Color( 132, 112, 255);
        private Color lightYellow = new Color( 255, 255, 224);
        private Color lightPink = new Color( 255, 160, 122 );
        private Color lightRed = new Color( 240, 128, 128 );
        private Color lightPurple = new Color( 147, 112, 219);
        private Color lightBrown = new Color( 139, 35, 35);
        private Color lightOrange = new Color( 255, 165, 0 );
        private Color lightGrey = new Color( 211, 211, 211 );
        public Paint getColor(VisualItem item) {
            if ( item instanceof EdgeItem ) {
            	Entity e = m_registry.getEntity( item );
            	HashSet fakeEdges = (HashSet) featurePairToFakeEdges.get( currentFeaturePair.toString() );
            	if( fakeEdges.contains( e )) 
            		return Color.WHITE;
            	else if ( item.isHighlighted() )
                    return pastelOrange;
                else
                    return Color.LIGHT_GRAY;
            } else {
            	if( hiddenItems.contains( item )) {
            		return Color.WHITE;
            	}
            	else
            		return Color.BLACK;
            }
        } //
       
        public Paint getFillColor(VisualItem item) {
        	if( item instanceof EdgeItem ) {
            	Entity e = m_registry.getEntity( item );
            	HashSet fakeEdges = (HashSet) featurePairToFakeEdges.get( currentFeaturePair.toString() );
            	if( fakeEdges.contains( e )) 
            		return Color.WHITE;
        	}
            if ( item.isHighlighted() && !(hiddenItems.contains(item))) {
            	return pastelOrange;            	
            }
            else if ( item instanceof NodeItem ) {
            	Entity e = m_registry.getEntity( item );
            	String label = ((Node) e).getAttribute("label");
            	//System.out.println( "Looking up " + label);
            //	System.out.println( "Looking up currentFeaturePair: " + currentFeaturePair);
            	HashMap m = (HashMap) userToCluster.get( currentFeaturePair.toString() );
            	//System.out.println( "Result = " + m);
            	Integer clusterId = (Integer) m.get( label );
            	if( highlightedUserList.contains( label) ) {
            		return Color.YELLOW;
            	}
            	if( hiddenItems.contains( item ))
            		return Color.WHITE;
            	
            	if( clusterId == null ) {
            		// user is not clustered
            		return Color.RED;
            	}
            	// System.out.println( "Result = " + clusterId );
            	int val = clusterId.intValue();
            	maxValue = new HashSet( m.values() ).size();
            	List l = new ArrayList( new HashSet( m.values() ) );
            	/*
            	if( maxValue == -1) {
            		Iterator iter = m.values().iterator();
            		while( iter.hasNext() ) {
            			Integer cId = (Integer) iter.next();
            			if( maxValue < cId.intValue() )
            				maxValue = cId.intValue();
            		}
            	}*/
            		
            	//System.out.println("maxValue = " + maxValue);
            //	System.out.println( "val = " + val + " scaled = " + ((val+1)*(255/(maxValue+1))));
            	int num = l.indexOf( new Integer(val))*(255/maxValue);
            	return new Color(num, num, 255 - num);
            	
            	/*
                if ( item.isFocus() )
                    return lightGreen; //pastelRed;
                else
                    return lightGray;
                  */  
            } else {
                return Color.BLACK;
            }
        } //        
    } //
    
} // end of class KirstenProject
