package prefuse.KirstenProject;


import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.List;
import java.util.ArrayList;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;


import edu.berkeley.guir.prefuse.graph.DefaultEdge;
import edu.berkeley.guir.prefuse.graph.Edge;
import edu.berkeley.guir.prefuse.graph.Graph;
import edu.berkeley.guir.prefuse.graph.DefaultNode;
import edu.berkeley.guir.prefuse.graph.DefaultGraph;
import edu.berkeley.guir.prefuse.graph.Node;

/**
 * krc: Modified to read in files in the following format:
 * <clusterings algorithm='foo' feature='bar'>
 * <clusters>
 *    <cluster>
 *      <user>a</user>
 *      <user>b</user>
 *    </cluster>
 * </clusters>
 * </clusterings>
 */
public class XMLClusterReader  {

	protected Class NODE_TYPE = DefaultNode.class;
	protected XMLClusterHandler handler;
	public HashMap getClusterIdToFeaturesMap() {
		return handler.clusterIdToFeaturesMap;
	}
	public HashMap getFeatureMap() {
		return handler.featureTypeMap;
	}
	public Pair loadClusters(InputStream is) throws IOException {
		try {		
			handler    = new XMLClusterHandler();
			SAXParserFactory factory   = SAXParserFactory.newInstance();
			SAXParser        saxParser = factory.newSAXParser();
			saxParser.parse(is, handler);
			return new Pair( handler.getUserToClusterMap(),
							handler.getClusterIdToUsersMap() );
		} catch ( SAXException se ) {
			se.printStackTrace();
		} catch ( ParserConfigurationException pce ) {
			pce.printStackTrace();
		} 
		return null;
	} //
	
	/**
	 * Return the class type used for representing nodes.
	 * @return the Class of new DefaultNode instances.
	 */
	public Class getNodeType() {
		return NODE_TYPE;
	} //
	
	/**
	 * Set the type to be used for node instances. All created nodes
	 * will be instances of the input type. The input class should be
	 * a descendant class of edu.berkeley.guir.prefuse.graph.DefaultNode.
	 * @param type the class type to use for node instances.
	 */
	public void setNodeType(Class type) {
		NODE_TYPE = type;
	} //
	
	/**
	 * Helper class that performs XML parsing of graph files using
	 * SAX (the Simple API for XML).
	 */
	public class XMLClusterHandler extends DefaultHandler {
		
		public static final String CLUSTERINGS = "clusterings";
		public static final String CLUSTERS   = "clusters";
		public static final String CLUSTER   = "cluster";
		public static final String USER    = "user";
		public static final String NAME		= "name";
		public static final String ALGORITHM = "algorithm";
		public static final String FEATURE = "feature";
		public static final String FEATURELIST = "featureList";
		public static final String FEATURETYPE = "featureType";
		
		protected HashMap featureAlgorithmPairToUserToClusterMap = new HashMap();
		protected HashMap featureAlgorithmPairToClusterIdToUsersMap = new HashMap();
		protected HashMap currentClusterIdToUserMap = new HashMap();
		protected HashMap currentUserToClusterIdMap = new HashMap();
		protected HashMap clusterIdToFeaturesMap = new HashMap();
		protected HashMap featureTypeMap = new HashMap();
		protected HashMap currentFeatureMap;
		protected String currentFeatureType;
		protected Pair featurePair;
		protected String currentFeatureName;
		protected List currentFeatureUsers;
		protected int nextId = 0;
		protected boolean insideFeatures = false;
		protected boolean insideFeatureType = false;
		
		public void startDocument() {
			featureAlgorithmPairToUserToClusterMap.clear();
			featureAlgorithmPairToClusterIdToUsersMap.clear();
			clusterIdToFeaturesMap.clear();
			featureTypeMap.clear();
			insideFeatures = false;
			nextId = 0;
		} //
		
		public void endElement(String namespaceURI, String localName, String qName) {
			if( qName.equals( FEATURELIST )) {
				insideFeatures = false;
			}
			if( qName.equals( FEATURE ) ) {
				insideFeatureType = false;
			}
		} //
		
		public void startElement(String namespaceURI, String localName, String qName, Attributes atts) {			
			if( qName.equals( CLUSTERS )) {
				featurePair = new Pair( atts.getValue( 0 ), atts.getValue( 1 ));
				currentClusterIdToUserMap = new HashMap();
				currentUserToClusterIdMap = new HashMap();
				//System.out.println( "Atts = " + atts.toString() );
				//System.out.println( "Setting up " + featurePair.toString());
				featureAlgorithmPairToUserToClusterMap.put ( featurePair.toString(), currentUserToClusterIdMap );
				featureAlgorithmPairToClusterIdToUsersMap.put( featurePair.toString(), currentClusterIdToUserMap );
			}
			if ( qName.equals(CLUSTER) ) {
				String pairString = (new Pair( new Integer(nextId), 
						featurePair.getSecond()).toString());
		    	//System.out.println( "Putting in features for " + pairString );
		    	clusterIdToFeaturesMap.put( pairString, new HashMap());
		    	
				currentClusterIdToUserMap.put( new Integer( nextId ), new ArrayList() );
				nextId++;
		    } else if ( qName.equals(USER) ) {
		    	if( insideFeatures ) 
		    	{
		    	//	System.out.println( "Parsing feature-user: " + atts.getValue(0));
		    		currentFeatureUsers.add( atts.getValue(0));
		    	}
		    	else if( insideFeatureType ) {
		    		List l = (List)currentFeatureMap.get( currentFeatureName );
		    		if( l == null )
		    			l = new ArrayList();
		    		l.add( atts.getValue(0));
		    		currentFeatureMap.put( currentFeatureName,l );
		    	}
		    	else {
		    	Integer curId = new Integer( nextId - 1 );
				List currentCluster = (List) currentClusterIdToUserMap.get( curId );
				String userName = atts.getValue( 0 );
				currentCluster.add( userName );
				currentClusterIdToUserMap.put( curId, currentCluster );
				currentUserToClusterIdMap.put( userName, curId );
		    	}
			} 
		    else if( qName.equals( FEATURELIST ) ) {
		    	System.out.println( "parsing feature: " + atts.getValue(0));
		    	currentFeatureUsers = new ArrayList();
		    	insideFeatures = true;
		    	Integer k = new Integer( nextId - 1);
		    	String pairString = (new Pair( k, featurePair.getSecond() ) ).toString();
		    	HashMap curMap = (HashMap) clusterIdToFeaturesMap.get( pairString );
		    	String currentFeatureType = (String)featurePair.getSecond();
		    	curMap.put( atts.getValue(0) + "," + currentFeatureType.toUpperCase(),
		    			currentFeatureUsers);	
		    }
		    else if( qName.equals( FEATURETYPE )) {
		    	currentFeatureMap = new HashMap();
		    	currentFeatureType = atts.getValue(0);
		    	featureTypeMap.put(currentFeatureType, currentFeatureMap );
		    }
		    else if( qName.equals( FEATURE ) ) {
		    	insideFeatureType = true;
		    	//System.out.println( "featureType feature = " + atts.getValue(0));
		    	currentFeatureName = atts.getValue(0);
		    }
		} //
		
		
		public HashMap getUserToClusterMap() {
			return featureAlgorithmPairToUserToClusterMap;
		} //
		public HashMap getClusterIdToUsersMap() {
			return featureAlgorithmPairToClusterIdToUsersMap;
		} //
	
	} // end of inner class XMLGraphHandler

} // end of class XMLGraphReader
